/*
	File:		HIDEmulation.c

	Contains:	HID Emulation glue code between the ADB SHIM and the USB Mouse HID Module

	Version:	xxx put version here xxx

	Copyright:	 1998-1999 by Apple Computer, Inc., all rights reserved.

*/

#include <Types.h>
#include <Devices.h>
#include <DriverServices.h>
#include <CursorDevices.h>
#include <USB.h>


#include "MouseModule.h"

extern	usbMousePBStruct myMousePB;

void USBMouseIn(UInt32 refcon, void * theData)
{
#pragma unused (refcon)

USBHIDDataPtr	pMouseData;
static UInt16 	oldbuttons = 0;
UInt16 	changedbuttons = 0;

//	DebugStr("\pIn USBMouseIn");
	pMouseData = (USBHIDDataPtr)theData;
	
	if (myMousePB.pCursorDeviceInfo)		// don't call cursor device manager functions without a good cursor device
	{
		// Tell the Cursor Device Manager that we moved
		if ((pMouseData->mouse.XDelta != 0) || (pMouseData->mouse.YDelta !=0))
		{
			CursorDeviceMove(myMousePB.pCursorDeviceInfo, pMouseData->mouse.XDelta, pMouseData->mouse.YDelta);
		}
		
		// Update with the state of the buttons.
		pMouseData->mouse.buttons &= 0x07;
		changedbuttons = oldbuttons ^ pMouseData->mouse.buttons;
		if (changedbuttons)
		{
			CursorDeviceButtons(myMousePB.pCursorDeviceInfo, (short)pMouseData->mouse.buttons);
		}
		oldbuttons = pMouseData->mouse.buttons;
	}
}

OSStatus USBHIDInstallInterrupt(HIDInterruptProcPtr HIDInterruptFunction, UInt32 refcon)
{
	USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Set interrupt pointer", (UInt32)HIDInterruptFunction);
	myMousePB.interruptRefcon = refcon;			
	
	if (HIDInterruptFunction == USBMouseIn)				// if someone is re-installing the USBMouseIn routine then this is
	{													// an indication that someone earlier requested the interrupt
		myMousePB.pSHIMInterruptRoutine = nil;			// pointer, saved it, and restored it.  When the USBMouseIn
		USBHIDControlDevice(kHIDEnableDemoMode, 0);		// routine is to be used, we've got to re-enable demo mode,
														// otherwise there won't be a cursor device manager entry for us...
	}
	else
	{
		USBHIDControlDevice(kHIDDisableDemoMode, 0);	// disable demo mode
		myMousePB.pSHIMInterruptRoutine = HIDInterruptFunction;
	}
	return noErr;
}

OSStatus USBHIDInstallNotification(HIDNotificationProcPtr HIDNotification, UInt32 refcon)
{
	USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Set notification proc ptr", (UInt32)HIDNotification);
	myMousePB.notificationRefcon = refcon;			
	
	myMousePB.pNotificationRoutine = HIDNotification;
	return noErr;
}

OSStatus USBHIDPollDevice(void)
{
	return kUSBInternalErr;
}

OSStatus USBHIDControlDevice(UInt32 theControlSelector, void * theControlData)
{
#pragma unused (theControlData)

	switch (theControlSelector)
	{
		case kHIDRemoveInterruptHandler:
			USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Removing interrupt handler!!!", myMousePB.interfaceRef);
			myMousePB.interruptRefcon = nil;
			myMousePB.pSavedInterruptRoutine = nil;
			myMousePB.pSHIMInterruptRoutine = nil;
			if (myMousePB.pCursorDeviceInfo != 0)
			{
				CursorDeviceDisposeDevice(myMousePB.pCursorDeviceInfo);
				myMousePB.pCursorDeviceInfo = 0;
			}
			break;
			
		case kHIDRemoveNotification:
			USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Removing notification!!!", myMousePB.interfaceRef);
			myMousePB.notificationRefcon = nil;
			myMousePB.pNotificationRoutine = NotifyRegisteredHIDUser;
			break;
			
		case kHIDEnableDemoMode:
			USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Demo Mode Enabled", myMousePB.interfaceRef);
			
			if (myMousePB.pCursorDeviceInfo == 0)
			{
				myMousePB.pCursorDeviceInfo = &myMousePB.cursorDeviceInfo;
				CursorDeviceNewDevice(&myMousePB.pCursorDeviceInfo);
				
				CursorDeviceSetAcceleration(myMousePB.pCursorDeviceInfo, (Fixed)(1<<16));
				
				CursorDeviceSetButtons(myMousePB.pCursorDeviceInfo, 3);			// should actually be set by reading
																				// the HID descriptor, but lacking
																				// a parser, we'll just force it
																				// this way.
				CursorDeviceButtonOp(myMousePB.pCursorDeviceInfo, 0, kButtonSingleClick, 0L);
				CursorDeviceButtonOp(myMousePB.pCursorDeviceInfo, 1, kButtonSingleClick, 0L);
				CursorDeviceButtonOp(myMousePB.pCursorDeviceInfo, 2, kButtonSingleClick, 0L);
				CursorDeviceUnitsPerInch(myMousePB.pCursorDeviceInfo, (Fixed)(myMousePB.unitsPerInch));
			
				myMousePB.pSavedInterruptRoutine = myMousePB.pSHIMInterruptRoutine;
				myMousePB.pSHIMInterruptRoutine = USBMouseIn;
			}
			break;

		case kHIDDisableDemoMode:
			USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Demo Mode Disabled", myMousePB.interfaceRef);
			if (myMousePB.pCursorDeviceInfo != 0)
			{
				CursorDeviceDisposeDevice(myMousePB.pCursorDeviceInfo);
				myMousePB.pCursorDeviceInfo = 0;
			}
			myMousePB.pSHIMInterruptRoutine = myMousePB.pSavedInterruptRoutine;
			break;

		default:
			return paramErr;
	}
	return noErr;
}



OSStatus USBHIDGetDeviceInfo(UInt32 theInfoSelector, void * theInfo)
{
HIDInterruptProcPtr * pHIDIntProcPtr;
UInt32 * pUnits;
UInt32 * pInterruptRefcon;

	switch (theInfoSelector)
	{
		case kHIDGetDeviceUnitsPerInch:
			pUnits = (UInt32*)theInfo;
			*pUnits = (UInt32)(myMousePB.unitsPerInch);
			break;
			
		case kHIDGetInterruptHandler:
			USBExpertStatus(myMousePB.interfaceRef, kMouseModuleName": Get interrupt pointer", (UInt32)(myMousePB.pSHIMInterruptRoutine));
			pHIDIntProcPtr = (HIDInterruptProcPtr *)theInfo;     
			*pHIDIntProcPtr = myMousePB.pSHIMInterruptRoutine;
			break;

		case kHIDGetInterruptRefcon:
			pInterruptRefcon = (UInt32 *)theInfo;
			*pInterruptRefcon = myMousePB.interruptRefcon;
			break;
		
		default:
			return paramErr;
	}
	return noErr;
}

OSStatus USBHIDEnterPolledMode(void)
{
	return unimpErr;
}

OSStatus USBHIDExitPolledMode(void)
{
	return unimpErr;
}

void NotifyRegisteredHIDUser(UInt32 refcon, UInt32 reportSize, void * theReport, USBReference theInterfaceRef)
//void NotifyRegisteredHIDUser(UInt32 devicetype, UInt8 hidReport[])
{
#pragma unused (refcon)
#pragma unused (reportSize)
#pragma unused (theInterfaceRef)

USBHIDData		theMouseData;
SInt8			myXDelta, myYDelta;
char * 			hidReport;

	hidReport = (char *)theReport;
	theMouseData.mouse.buttons = (UInt16)hidReport[0];
	
	myXDelta = (SInt8)hidReport[1];
	myYDelta = (SInt8)hidReport[2];  
	
	theMouseData.mouse.XDelta = (SInt16)myXDelta;
	theMouseData.mouse.YDelta = (SInt16)myYDelta;
	if (myMousePB.pSHIMInterruptRoutine)
	{
		(*myMousePB.pSHIMInterruptRoutine)(myMousePB.interruptRefcon, (void *)&theMouseData);
	}
}

USBHIDRev2DispatchTable TheHIDModuleDispatchTable =
{
	(UInt32)kHIDCurrentDispatchVersion,
	(USBHIDInstallInterruptProcPtr)USBHIDInstallInterrupt,
	(USBHIDPollDeviceProcPtr)USBHIDPollDevice,
	(USBHIDControlDeviceProcPtr)USBHIDControlDevice,
	(USBHIDGetDeviceInfoProcPtr)USBHIDGetDeviceInfo,
	(USBHIDEnterPolledModeProcPtr)USBHIDEnterPolledMode,
	(USBHIDExitPolledModeProcPtr)USBHIDExitPolledMode,
	(USBHIDInstallNotificationProcPtr)USBHIDInstallNotification
};

CursorDevicePtr gUSBMouse;
CursorDevice	ourDevice;

